@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: util.c,v 1.153 2011/05/07 16:05:17 rbrito Exp $ */
+/* $Id: util.c,v 1.154 2011/10/18 21:51:20 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -808,7 +808,7 @@ has_SSE(void)
 #ifdef HAVE_NASM
     return has_SSE_nasm();
 #else
-#ifdef _M_X64
+#if defined( _M_X64 ) || defined( MIN_ARCH_SSE )
     return 1;
 #else
     return 0;           /* don't know, assume not */
@@ -822,7 +822,7 @@ has_SSE2(void)
 #ifdef HAVE_NASM
     return has_SSE2_nasm();
 #else
-#ifdef _M_X64
+#if defined( _M_X64 ) || defined( MIN_ARCH_SSE )
     return 1;
 #else
     return 0;           /* don't know, assume not */
