@@ -1,10 +1,9 @@
 /* 
- * Mpeg Layer-3 audio decoder 
- * --------------------------
- * copyright (c) 1995,1996,1997 by Michael Hipp.
- * All rights reserved. See also 'README'
+ * layer3.c: Mpeg Layer-3 audio decoder 
  *
- * Copyright (C) 2000 Albert L. Faber
+ * Copyright (C) 1999-2010 The L.A.M.E. project
+ *
+ * Initially written by Michael Hipp, see also AUTHORS and README.
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,8 +20,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-/* $Id: layer3.c,v 1.51.2.2 2008/10/23 23:35:54 robert Exp $ */
+/* $Id: layer3.c,v 1.63 2011/03/05 14:06:54 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -43,6 +41,7 @@
 #endif
 
 
+static int gd_are_hip_tables_layer3_initialized = 0;
 
 static real ispow[8207];
 static real aa_ca[8], aa_cs[8];
@@ -67,7 +66,7 @@ static int shortLimit[9][14];
 
 /* *INDENT-OFF* */
 
-const struct bandInfoStruct bandInfo[9] = { 
+static const struct bandInfoStruct bandInfo[9] = { 
 
 /* MPEG 1.0 */
  { {0,4,8,12,16,20,24,30,36,44,52,62,74, 90,110,134,162,196,238,288,342,418,576},
@@ -148,10 +147,15 @@ get1bit(PMPSTR mp)
  * init tables for layer-3 
  */
 void
-init_layer3(int down_sample_sblimit)
+hip_init_tables_layer3(void)
 {
     int     i, j, k;
 
+    if (gd_are_hip_tables_layer3_initialized) {
+        return;
+    }
+    gd_are_hip_tables_layer3_initialized = 1;
+
     for (i = -256; i < 118 + 4; i++)
         gainpow2[i + 256] = pow((double) 2.0, -0.25 * (double) (i + 210));
 
@@ -207,7 +211,7 @@ init_layer3(int down_sample_sblimit)
     }
 
     for (j = 0; j < 4; j++) {
-        static int len[4] = { 36, 36, 12, 36 };
+        static int const len[4] = { 36, 36, 12, 36 };
         for (i = 0; i < len[j]; i += 2)
             win1[j][i] = +win[j][i];
         for (i = 1; i < len[j]; i += 2)
@@ -291,13 +295,13 @@ init_layer3(int down_sample_sblimit)
     for (j = 0; j < 9; j++) {
         for (i = 0; i < 23; i++) {
             longLimit[j][i] = (bandInfo[j].longIdx[i] - 1 + 8) / 18 + 1;
-            if (longLimit[j][i] > (down_sample_sblimit))
-                longLimit[j][i] = down_sample_sblimit;
+            if (longLimit[j][i] > SBLIMIT)
+                longLimit[j][i] = SBLIMIT;
         }
         for (i = 0; i < 14; i++) {
             shortLimit[j][i] = (bandInfo[j].shortIdx[i] - 1) / 18 + 1;
-            if (shortLimit[j][i] > (down_sample_sblimit))
-                shortLimit[j][i] = down_sample_sblimit;
+            if (shortLimit[j][i] > SBLIMIT)
+                shortLimit[j][i] = SBLIMIT;
         }
     }
 
@@ -351,31 +355,31 @@ init_layer3(int down_sample_sblimit)
  */
 
 static void
-III_get_side_info_1(PMPSTR mp, struct III_sideinfo *si, int stereo,
+III_get_side_info_1(PMPSTR mp, int stereo,
                     int ms_stereo, long sfreq, int single)
 {
     int     ch, gr;
     int     powdiff = (single == 3) ? 4 : 0;
 
-    si->main_data_begin = getbits(mp, 9);
+    mp->sideinfo.main_data_begin = getbits(mp, 9);
     if (stereo == 1)
-        si->private_bits = getbits_fast(mp, 5);
+        mp->sideinfo.private_bits = getbits_fast(mp, 5);
     else
-        si->private_bits = getbits_fast(mp, 3);
+        mp->sideinfo.private_bits = getbits_fast(mp, 3);
 
     for (ch = 0; ch < stereo; ch++) {
-        si->ch[ch].gr[0].scfsi = -1;
-        si->ch[ch].gr[1].scfsi = getbits_fast(mp, 4);
+        mp->sideinfo.ch[ch].gr[0].scfsi = -1;
+        mp->sideinfo.ch[ch].gr[1].scfsi = getbits_fast(mp, 4);
     }
 
     for (gr = 0; gr < 2; gr++) {
         for (ch = 0; ch < stereo; ch++) {
-            struct gr_info_s *gr_infos = &(si->ch[ch].gr[gr]);
+            struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[gr]);
 
             gr_infos->part2_3_length = getbits(mp, 12);
             gr_infos->big_values = getbits_fast(mp, 9);
             if (gr_infos->big_values > 288) {
-                fprintf(stderr, "big_values too large! %i\n", gr_infos->big_values);
+                lame_report_fnc(mp->report_err, "big_values too large! %i\n", gr_infos->big_values);
                 gr_infos->big_values = 288;
             }
             {
@@ -410,7 +414,7 @@ III_get_side_info_1(PMPSTR mp, struct III_sideinfo *si, int stereo,
                 }
 
                 if (gr_infos->block_type == 0) {
-                    fprintf(stderr, "Blocktype == 0 and window-switching == 1 not allowed.\n");
+                    lame_report_fnc(mp->report_err, "Blocktype == 0 and window-switching == 1 not allowed.\n");
                     /* error seems to be very good recoverable, so don't exit */
                     /* exit(1); */
                 }
@@ -419,13 +423,23 @@ III_get_side_info_1(PMPSTR mp, struct III_sideinfo *si, int stereo,
                 gr_infos->region2start = 576 >> 1;
             }
             else {
-                int     i, r0c, r1c;
+                unsigned int i, r0c, r1c, region0index, region1index;
                 for (i = 0; i < 3; i++)
                     gr_infos->table_select[i] = getbits_fast(mp, 5);
                 r0c = getbits_fast(mp, 4);
                 r1c = getbits_fast(mp, 3);
-                gr_infos->region1start = bandInfo[sfreq].longIdx[r0c + 1] >> 1;
-                gr_infos->region2start = bandInfo[sfreq].longIdx[r0c + 1 + r1c + 1] >> 1;
+                region0index = r0c+1;
+                if (region0index > 22) {
+                    region0index = 22;
+                    lame_report_fnc(mp->report_err, "region0index > 22\n");
+                }
+                region1index = r0c+1 + r1c+1;
+                if (region1index > 22) {
+                    region1index = 22;
+                    lame_report_fnc(mp->report_err, "region1index > 22\n");
+                }
+                gr_infos->region1start = bandInfo[sfreq].longIdx[region0index] >> 1;
+                gr_infos->region2start = bandInfo[sfreq].longIdx[region1index] >> 1;
                 gr_infos->block_type = 0;
                 gr_infos->mixed_block_flag = 0;
             }
@@ -440,27 +454,26 @@ III_get_side_info_1(PMPSTR mp, struct III_sideinfo *si, int stereo,
  * Side Info for MPEG 2.0 / LSF
  */
 static void
-III_get_side_info_2(PMPSTR mp, struct III_sideinfo *si, int stereo,
-                    int ms_stereo, long sfreq, int single)
+III_get_side_info_2(PMPSTR mp, int stereo, int ms_stereo, long sfreq, int single)
 {
     int     ch;
     int     powdiff = (single == 3) ? 4 : 0;
 
-    si->main_data_begin = getbits(mp, 8);
+    mp->sideinfo.main_data_begin = getbits(mp, 8);
 
     if (stereo == 1)
-        si->private_bits = get1bit(mp);
+        mp->sideinfo.private_bits = get1bit(mp);
     else
-        si->private_bits = getbits_fast(mp, 2);
+        mp->sideinfo.private_bits = getbits_fast(mp, 2);
 
     for (ch = 0; ch < stereo; ch++) {
-        struct gr_info_s *gr_infos = &(si->ch[ch].gr[0]);
+        struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[0]);
         unsigned int qss;
 
         gr_infos->part2_3_length = getbits(mp, 12);
         gr_infos->big_values = getbits_fast(mp, 9);
         if (gr_infos->big_values > 288) {
-            fprintf(stderr, "big_values too large! %i\n", gr_infos->big_values);
+            lame_report_fnc(mp->report_err, "big_values too large! %i\n", gr_infos->big_values);
             gr_infos->big_values = 288;
         }
         qss = getbits_fast(mp, 8);
@@ -494,7 +507,7 @@ III_get_side_info_2(PMPSTR mp, struct III_sideinfo *si, int stereo,
             }
 
             if (gr_infos->block_type == 0) {
-                fprintf(stderr, "Blocktype == 0 and window-switching == 1 not allowed.\n");
+                lame_report_fnc(mp->report_err, "Blocktype == 0 and window-switching == 1 not allowed.\n");
                 /* error seems to be very good recoverable, so don't exit */
                 /* exit(1); */
             }
@@ -514,13 +527,23 @@ III_get_side_info_2(PMPSTR mp, struct III_sideinfo *si, int stereo,
             gr_infos->region2start = 576 >> 1;
         }
         else {
-            int     i, r0c, r1c;
+            unsigned int i, r0c, r1c, region0index, region1index;
             for (i = 0; i < 3; i++)
                 gr_infos->table_select[i] = getbits_fast(mp, 5);
             r0c = getbits_fast(mp, 4);
             r1c = getbits_fast(mp, 3);
-            gr_infos->region1start = bandInfo[sfreq].longIdx[r0c + 1] >> 1;
-            gr_infos->region2start = bandInfo[sfreq].longIdx[r0c + 1 + r1c + 1] >> 1;
+            region0index = r0c+1;
+            if (region0index > 22) {
+                region0index = 22;
+                lame_report_fnc(mp->report_err, "region0index > 22\n");
+            }
+            region1index = r0c+1 + r1c+1;
+            if (region1index > 22) {
+                region1index = 22;
+                lame_report_fnc(mp->report_err, "region1index > 22\n");
+            }
+            gr_infos->region1start = bandInfo[sfreq].longIdx[region0index] >> 1;
+            gr_infos->region2start = bandInfo[sfreq].longIdx[region1index] >> 1;
             gr_infos->block_type = 0;
             gr_infos->mixed_block_flag = 0;
         }
@@ -693,7 +716,7 @@ III_dequantize_sample(PMPSTR mp, real xr[SBLIMIT][SSLIMIT], int *scf,
     int     part2remain = gr_infos->part2_3_length - part2bits;
     int    *me;
 
-    /* fprintf(stderr,"part2remain = %d, gr_infos->part2_3_length = %d, part2bits = %d\n",
+    /* lame_report_fnc(mp->report_dbg,"part2remain = %d, gr_infos->part2_3_length = %d, part2bits = %d\n",
        part2remain, gr_infos->part2_3_length, part2bits); */
 
     {
@@ -739,7 +762,7 @@ III_dequantize_sample(PMPSTR mp, real xr[SBLIMIT][SSLIMIT], int *scf,
         int     i;
         for (i = 0; i < 3; i++) {
             if (l[i] < 0) {
-                fprintf(stderr, "hip: Bogus region length (%d)\n", l[i]);
+                lame_report_fnc(mp->report_err, "hip: Bogus region length (%d)\n", l[i]);
                 l[i] = 0;
             }
         }
@@ -1066,7 +1089,7 @@ III_dequantize_sample(PMPSTR mp, real xr[SBLIMIT][SSLIMIT], int *scf,
     if (part2remain > 0)
         getbits(mp, part2remain);
     else if (part2remain < 0) {
-        fprintf(stderr, "hip: Can't rewind stream by %d bits!\n", -part2remain);
+        lame_report_fnc(mp->report_err, "hip: Can't rewind stream by %d bits!\n", -part2remain);
         return 1;       /* -> error */
     }
     return 0;
@@ -1567,7 +1590,6 @@ III_hybrid(PMPSTR mp, real fsIn[SBLIMIT][SSLIMIT], real tsOut[SSLIMIT][SBLIMIT],
 /*
  * main layer3 handler
  */
-struct III_sideinfo sideinfo;
 
 int
 layer3_audiodata_precedesframes(PMPSTR mp)
@@ -1577,18 +1599,18 @@ layer3_audiodata_precedesframes(PMPSTR mp)
 
     /* specific to Layer 3, since Layer 1 & 2 the audio data starts at the frame that describes it. */
     /* determine how many bytes and therefore bitstream frames the audio data precedes it's matching frame */
-    /* fprintf(stderr, "hip: main_data_begin = %d, mp->bsize %d, mp->fsizeold %d, mp->ssize %d\n",
+    /* lame_report_fnc(mp->report_err, "hip: main_data_begin = %d, mp->bsize %d, mp->fsizeold %d, mp->ssize %d\n",
        sideinfo.main_data_begin, mp->bsize, mp->fsizeold, mp->ssize); */
     /* compute the number of frames to backtrack, 4 for the header, ssize already holds the CRC */
     /* TODO Erroneously assumes current frame is same as previous frame. */
     audioDataInFrame = mp->bsize - 4 - mp->ssize;
-    framesToBacktrack = (sideinfo.main_data_begin + audioDataInFrame - 1) / audioDataInFrame;
-    /* fprintf(stderr, "hip: audioDataInFrame %d framesToBacktrack %d\n", audioDataInFrame, framesToBacktrack); */
+    framesToBacktrack = (mp->sideinfo.main_data_begin + audioDataInFrame - 1) / audioDataInFrame;
+    /* lame_report_fnc(mp->report_err, "hip: audioDataInFrame %d framesToBacktrack %d\n", audioDataInFrame, framesToBacktrack); */
     return framesToBacktrack;
 }
 
 int
-do_layer3_sideinfo(PMPSTR mp)
+decode_layer3_sideinfo(PMPSTR mp)
 {
     struct frame *fr = &mp->fr;
     int     stereo = fr->stereo;
@@ -1611,27 +1633,27 @@ do_layer3_sideinfo(PMPSTR mp)
 
     if (fr->lsf) {
         granules = 1;
-        III_get_side_info_2(mp, &sideinfo, stereo, ms_stereo, sfreq, single);
+        III_get_side_info_2(mp, stereo, ms_stereo, sfreq, single);
     }
     else {
         granules = 2;
-        III_get_side_info_1(mp, &sideinfo, stereo, ms_stereo, sfreq, single);
+        III_get_side_info_1(mp, stereo, ms_stereo, sfreq, single);
     }
 
     databits = 0;
     for (gr = 0; gr < granules; ++gr) {
         for (ch = 0; ch < stereo; ++ch) {
-            struct gr_info_s *gr_infos = &(sideinfo.ch[ch].gr[gr]);
+            struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[gr]);
             databits += gr_infos->part2_3_length;
         }
     }
-    return databits - 8 * sideinfo.main_data_begin;
+    return databits - 8 * mp->sideinfo.main_data_begin;
 }
 
 
 
 int
-do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
+decode_layer3_frame(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
           int (*synth_1to1_mono_ptr) (PMPSTR, real *, unsigned char *, int *),
           int (*synth_1to1_ptr) (PMPSTR, real *, int, unsigned char *, int *))
 {
@@ -1645,7 +1667,7 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
     int     sfreq = fr->sampling_frequency;
     int     stereo1, granules;
 
-    if (set_pointer(mp, (int) sideinfo.main_data_begin) == MP3_ERR)
+    if (set_pointer(mp, (int) mp->sideinfo.main_data_begin) == MP3_ERR)
         return 0;
 
     if (stereo == 1) {  /* stream is mono */
@@ -1677,7 +1699,7 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
         static real hybridOut[2][SSLIMIT][SBLIMIT];
 
         {
-            struct gr_info_s *gr_infos = &(sideinfo.ch[0].gr[gr]);
+            struct gr_info_s *gr_infos = &(mp->sideinfo.ch[0].gr[gr]);
             long    part2bits;
 
             if (fr->lsf)
@@ -1693,12 +1715,12 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
                     mp->pinfo->sfb_s[gr][0][i] = scalefacs[0][i];
             }
 
-            /* fprintf(stderr, "calling III dequantize sample 1 gr_infos->part2_3_length %d\n", gr_infos->part2_3_length); */
+            /* lame_report_fnc(mp->report_err, "calling III dequantize sample 1 gr_infos->part2_3_length %d\n", gr_infos->part2_3_length); */
             if (III_dequantize_sample(mp, hybridIn[0], scalefacs[0], gr_infos, sfreq, part2bits))
                 return clip;
         }
         if (stereo == 2) {
-            struct gr_info_s *gr_infos = &(sideinfo.ch[1].gr[gr]);
+            struct gr_info_s *gr_infos = &(mp->sideinfo.ch[1].gr[gr]);
             long    part2bits;
             if (fr->lsf)
                 part2bits = III_get_scale_factors_2(mp, scalefacs[1], gr_infos, i_stereo);
@@ -1712,7 +1734,7 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
                     mp->pinfo->sfb_s[gr][1][i] = scalefacs[1][i];
             }
 
-            /* fprintf(stderr, "calling III dequantize sample 2  gr_infos->part2_3_length %d\n", gr_infos->part2_3_length); */
+            /* lame_report_fnc(mp->report_err, "calling III dequantize sample 2  gr_infos->part2_3_length %d\n", gr_infos->part2_3_length); */
             if (III_dequantize_sample(mp, hybridIn[1], scalefacs[1], gr_infos, sfreq, part2bits))
                 return clip;
 
@@ -1731,10 +1753,10 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
                 III_i_stereo(hybridIn, scalefacs[1], gr_infos, sfreq, ms_stereo, fr->lsf);
 
             if (ms_stereo || i_stereo || (single == 3)) {
-                if (gr_infos->maxb > sideinfo.ch[0].gr[gr].maxb)
-                    sideinfo.ch[0].gr[gr].maxb = gr_infos->maxb;
+                if (gr_infos->maxb > mp->sideinfo.ch[0].gr[gr].maxb)
+                    mp->sideinfo.ch[0].gr[gr].maxb = gr_infos->maxb;
                 else
-                    gr_infos->maxb = sideinfo.ch[0].gr[gr].maxb;
+                    gr_infos->maxb = mp->sideinfo.ch[0].gr[gr].maxb;
             }
 
             switch (single) {
@@ -1770,10 +1792,10 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
             mp->pinfo->js = (fr->mode == MPG_MD_JOINT_STEREO);
             mp->pinfo->ms_stereo = ms_stereo;
             mp->pinfo->i_stereo = i_stereo;
-            mp->pinfo->maindata = sideinfo.main_data_begin;
+            mp->pinfo->maindata = mp->sideinfo.main_data_begin;
 
             for (ch = 0; ch < stereo1; ch++) {
-                struct gr_info_s *gr_infos = &(sideinfo.ch[ch].gr[gr]);
+                struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[gr]);
                 mp->pinfo->big_values[gr][ch] = gr_infos->big_values;
                 mp->pinfo->scalefac_scale[gr][ch] = gr_infos->scalefac_scale;
                 mp->pinfo->mixed[gr][ch] = gr_infos->mixed_block_flag;
@@ -1786,7 +1808,7 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
 
 
             for (ch = 0; ch < stereo1; ch++) {
-                struct gr_info_s *gr_infos = &(sideinfo.ch[ch].gr[gr]);
+                struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[gr]);
                 ifqstep = (mp->pinfo->scalefac_scale[gr][ch] == 0) ? .5 : 1.0;
                 if (2 == gr_infos->block_type) {
                     for (i = 0; i < 3; i++) {
@@ -1828,7 +1850,7 @@ do_layer3(PMPSTR mp, unsigned char *pcm_sample, int *pcm_point,
 
 
         for (ch = 0; ch < stereo1; ch++) {
-            struct gr_info_s *gr_infos = &(sideinfo.ch[ch].gr[gr]);
+            struct gr_info_s *gr_infos = &(mp->sideinfo.ch[ch].gr[gr]);
             III_antialias(hybridIn[ch], gr_infos);
             III_hybrid(mp, hybridIn[ch], hybridOut[ch], ch, gr_infos);
         }
